// EEPROM_SPI driver

// ***************************************************************
// *               Copyright (C) Frank Linder                    *
// * Permission is granted to use, modify, or redistribute this  *
// * software so long as it is not sold or exploited for profit. *
// *    Provided "as is" without express or implied warranty.    *
// ***************************************************************

#include "main.h"
#include "spi_mc.h"
#include "EEPROMspi.h"                                    

eedata_Write *eepspi_wr_desc;

// SPI EEPROM-port description
rom spi_datastruct eeprom_desc=
	{&PORTA, &DDRA, MODE_00, SPI_FAST , 0x20, 0x01, SMPMID }; //SPI_EEPROM,bit 5


//----- functions ----------------------------------------
void				eepspi_init(void) {
	
	//PortA bit 5
	PORTAbits.RA5=1;	// normally high
	DDRAbits.RA5=0;	// bit 5 = output
	
}
//--------------------------------------------------------

void 	eepspi_open(eedata_Write *wr_desc, unsigned long addr, unsigned char adrbus) {
	if( spi_isopen() ) spi_close();   
   eepspi_wr_desc=wr_desc;   
   
	if(eepspi_wr_desc) {
	   eepspi_wr_desc->addr=addr;
	   eepspi_wr_desc->adrbus=adrbus&63;

		spi_open(&eeprom_desc);	
		spi_write(EESPIWrEn);   
		spi_close();		
	}
	
	spi_open(&eeprom_desc);
	if(eepspi_wr_desc) spi_write(EESPIWr);   
	else               spi_write(EESPIRd);   
		
	if(adrbus==24u)	spi_write(addr>>16u);
	spi_write(addr>>8u);
	spi_write(addr);      		
}
//--------------------------------------------------------

void 	eepspi_close(void) {
	if(eepspi_wr_desc->adrbus&128u) {
		while(1) {
			spi_close(); 
			spi_open(&eeprom_desc);				
			spi_write(EESPIRdSR);   
			if((spi_read() & 1)==0u) break;
		}			
	}	            
	spi_close(); 			
}
//--------------------------------------------------------

void eepspi_put (unsigned char c) { 
	eepspi_wr_desc->addr++;
	eepspi_wr_desc->adrbus|=128u;
	spi_write(c);
	if( (eepspi_wr_desc->addr & (eepspi_wr_desc->page - 1u))==0u ) {
		eepspi_close();
		eepspi_open(eepspi_wr_desc, eepspi_wr_desc->addr, eepspi_wr_desc->adrbus );				
	}	
}
//--------------------------------------------------------

void eepspi_erase( void ) {
	if( spi_isopen() ) return;
	spi_open(&eeprom_desc);	
	spi_write(EESPIErAll);   
	spi_close(); 
	
	spi_open(&eeprom_desc);				
	while(spi_read() & 1); // if programming is in progress
	spi_close(); 
}
//--------------------------------------------------------

void eepspi_write ( unsigned char *p, unsigned int num ) {
	while(num) {
		eepspi_put(*p);
		p++;
		num--;
	}		
}
//--------------------------------------------------------

void eepspi_read ( unsigned char *p, unsigned int num ) {	  
	if(eepspi_wr_desc) return;
	while(num) {
		*p=spi_read();
		p++;
		num--;
	}
}		
//--------------------------------------------------------

unsigned int eepspi_puts (unsigned char *p) {
	unsigned int i=0;	
	while (1u) {
		i++;
		eepspi_put(*p);
		if(*p==0u) break;
		p++;
	} 
	return i;
}
//--------------------------------------------------------

unsigned int eepspi_gets (unsigned char *p, unsigned int num) {
	unsigned int i=0;
	if(eepspi_wr_desc) return 0;

	while (1u) {
		i++;
		*p=spi_read();
		if( i>=num ) {
			*p=0;			
		}			
		if(*p==0u) break;
		p++;
	}			
	return i;
}
//--------------------------------------------------------


