
// ***************************************************************
// *               Copyright (C) Frank Linder                    *
// * Permission is granted to use, modify, or redistribute this  *
// * software so long as it is not sold or exploited for profit. *
// *    Provided "as is" without express or implied warranty.    *
// ***************************************************************


#include "main.h"
#include "spi_mc.h"
#include "glcd.h"
#include "Font_8x6.c"

//----- defines ------------------------------------------


//----- variables ----------------------------------------
unsigned char glcd_in_use; // 0=not in use 
									//	1=in use shiftreg random
									//	2=in use shiftreg&glcd_hw_cfg=write_data

bituchar 	  glcd_hw_cfg; 
unsigned char glcd_px;	 	//	(disp 128x64) 0-127
unsigned char glcd_py;	 	// byte addres (disp 128x64) 0-7
unsigned char glcd_clipx;
unsigned char glcd_clipy;
unsigned char glcd_clipx2;
unsigned char glcd_clipy2;
unsigned char glcd_ww;
									
unsigned char glcd_gmode;
unsigned char glcd_gmodemask;

rom unsigned char glcd_pixtab[8]={1,2,4,8,16,32,64,128};
	
rom spi_datastruct G_disp_desc=
	{&PORTE, &DDRE, MODE_00, SPI_ULTRA, 0x01, 0x08, SMPMID };//SPI_CH_DISPLAY,bit 0	

//----- functions prototypes------------------------------
	void 				glcd_write_com(unsigned char byte);
	void 				glcd_write_data(unsigned char byte);
	unsigned char	glcd_read_com(void);
	unsigned char	glcd_read_data(void);
	void				glcd_ch_chip(void);


//----- functions ----------------------------------------
void 				glcd_init(void) {
	PORTEbits.RE0=0;  // normally low
	DDREbits.RE0=0;
	GLCD_TRIS_PORT = 0;
	
	Delay10KTCYx(64);
	
		glcd_hw_cfg.data=0b00000000;
		glcd_in_use=0;
		glcd_gmode=GLCD_B_W;
#ifndef GLCD_inverted
		glcd_gmodemask=0;
#endif
#ifdef GLCD_inverted
		glcd_gmodemask=255;
#endif
	
	glcd_clipx=0;
	glcd_clipy=0;

	#ifdef GLCD_Display128x64
		glcd_clipx2=128;
		glcd_clipy2=8;
	#endif
	#ifdef GLCD_Display256x64
		glcd_clipx2=256;
		glcd_clipy2=8;
	#endif
	#ifdef GLCD_Display128x128
		glcd_clipx2=128;
		glcd_clipy2=16;
	#endif

	glcd_clear_all();

//	glcd_chip=0; // no select
}
//--------------------------------------------------------

unsigned char	glcd_open(void) {
	if(glcd_in_use) return 0;
	if( spi_isopen() ) return 0;
	spi_open(&G_disp_desc);	

	glcd_in_use=1;	// display in use
	glcd_hw_cfg.data|=GLCD_SELBITS_SELECT;
	//glcd_write_com(GLCD_ON);
	glcd_write_com(GLCD_ON);
	return 1;
}
//--------------------------------------------------------

void 				glcd_close(void) {
	spi_close();
	glcd_in_use=0u;
}
//--------------------------------------------------------

void 				glcd_write_com(unsigned char byte){	
	if(glcd_in_use==0u) return;
#ifdef GLCD_UseBusyFlag
	while(glcd_read_com()&GLCD_BUSYB) ;
#endif	
	glcd_in_use=1;	
	glcd_hw_cfg.GLCD_RS=0;
	glcd_hw_cfg.GLCD_RW=0;
	GLCD_TRIS_PORT=0;
	spi_write(glcd_hw_cfg.data);
	GLCD_DATA_PORT=0;
	GLCD_DATA_PORT=1;
	GLCD_DATA_PORT=byte;
	spi_strobe(SPI_STROBE);
}
//--------------------------------------------------------

void 				glcd_write_data(unsigned char byte){
	if(glcd_in_use==0u) return;
#ifdef GLCD_UseBusyFlag
	while(glcd_read_com()&GLCD_BUSYB) ;
#endif
	if(glcd_in_use!=2u) {	
		glcd_hw_cfg.GLCD_RS=1;
		glcd_hw_cfg.GLCD_RW=0;
		GLCD_TRIS_PORT=0;
		spi_write(glcd_hw_cfg.data);
		GLCD_DATA_PORT=0;
		GLCD_DATA_PORT=1;
		glcd_in_use=2;
	}
	GLCD_DATA_PORT=byte;
	glcd_px++;
	spi_strobe(SPI_STROBE);
	if((glcd_px&63)==0u){
		glcd_ch_chip();
		glcd_in_use=1;
	}	
}
//--------------------------------------------------------

unsigned char	glcd_read_com(void){
	unsigned char retval;
	if(glcd_in_use==0u) return 0;
	glcd_in_use=1;	
	glcd_hw_cfg.GLCD_RS=0;
	glcd_hw_cfg.GLCD_RW=1;
	GLCD_TRIS_PORT=0;
	GLCD_DATA_PORT=0;
	spi_write(glcd_hw_cfg.data);
	GLCD_DATA_PORT=0;
	GLCD_DATA_PORT=1;
	GLCD_TRIS_PORT=255;

	spi_strobe(SPI_STROBE_SET);	
	retval=GLCD_DATA_PORT;
	spi_strobe(SPI_STROBE_RST);		
	return retval;
}
//--------------------------------------------------------

unsigned char	glcd_read_data(void){
	unsigned char retval;
	if(glcd_in_use==0u) return 0;
#ifdef GLCD_UseBusyFlag
	while(glcd_read_com()&GLCD_BUSYB) ;
#endif
	glcd_in_use=1;	
	glcd_hw_cfg.GLCD_RS=1;
	glcd_hw_cfg.GLCD_RW=1;
	GLCD_TRIS_PORT=0;
	GLCD_DATA_PORT=0;
	spi_write(glcd_hw_cfg.data);
	glcd_px++;
	GLCD_DATA_PORT=1;
	if(!(glcd_px&63)){
		glcd_ch_chip();
	}	
	GLCD_DATA_PORT=1;
	GLCD_TRIS_PORT=255;
	spi_strobe(SPI_STROBE);	

	spi_strobe(SPI_STROBE_SET);	
	retval=GLCD_DATA_PORT;
	spi_strobe(SPI_STROBE_RST);		
	return retval;
}
//--------------------------------------------------------

void	glcd_write(unsigned char byte){
	glcd_write_data(byte^glcd_gmodemask);
}
//--------------------------------------------------------

void 	glcd_xy(unsigned char x, unsigned char y) {
// y=byte addres
	if(glcd_in_use==0u) return;

	glcd_px=x;
	glcd_py=y;

	glcd_hw_cfg.data|=GLCD_SELBITS_SELECT;

	glcd_write_com((y&7)|GLCD_YPOS);
	glcd_write_com(GLCD_XPOS);

	#ifdef GLCD_Display128x64
		if(y>7u) return;
		if(x>127u) return;
		glcd_hw_cfg.GLCD_SEL0=1;
		glcd_hw_cfg.GLCD_SEL1=0;
		if(x>63u) {
			glcd_hw_cfg.GLCD_SEL0=0;
			glcd_hw_cfg.GLCD_SEL1=1;
		}
	#endif
	#ifdef GLCD_Display256x64
		if(y>7u) return;
//		if(x>239) return;

		glcd_hw_cfg.data&=GLCD_SELBITS_MASK;
		if(x>191u) {
			glcd_hw_cfg.GLCD_SEL3=1;
		}
		else if(x>127u) {
			glcd_hw_cfg.GLCD_SEL2=1;
		}
		else if(x>63u) {
			glcd_hw_cfg.GLCD_SEL1=1;
		}
		else {
			glcd_hw_cfg.GLCD_SEL0=1;
		}
	#endif
	#ifdef GLCD_Displayd128x128
		if(y>15u) return;
		if(x>127u) return;

		glcd_hw_cfg.data|=GLCD_SELBITS_SELECT;

		if(x>63u) {
			glcd_hw_cfg.GLCD_SEL0=0;
			glcd_hw_cfg.GLCD_SEL2=0;
		}
		else {
			glcd_hw_cfg.GLCD_SEL1=0;
			glcd_hw_cfg.GLCD_SEL3=0;
		}

		if(y>7u) {
			glcd_hw_cfg.GLCD_SEL0=0;
			glcd_hw_cfg.GLCD_SEL1=0;
		}
		else {
			glcd_hw_cfg.GLCD_SEL2=0;
			glcd_hw_cfg.GLCD_SEL3=0;
		}
	#endif

	glcd_write_com((x&63)|GLCD_XPOS);	
}
//--------------------------------------------------------

void 	glcd_puts_rom(rom const char *s) {
	if(glcd_in_use==0u) return;
	while(*s){
		glcd_print_char(*(s++) );
	}
}
//--------------------------------------------------------

void 	glcd_puts(char *s) {
	if(glcd_in_use==0u) return;
	while(*s){
		glcd_print_char(*(s++));
	}
}
//--------------------------------------------------------

void 	glcd_draw_xy_rom(rom const unsigned char *img, 
												unsigned char x, unsigned char y) {
// y=byte addres
	unsigned char w,h;

	if(glcd_in_use==0u) return;
	w=*(img++);
	h=*(img++);
	
	do{
		glcd_xy(x,y++);
		glcd_ww=w;
		do{
			glcd_write_data(*(img++) ^ glcd_gmodemask);
		} while(--glcd_ww);
	} while(--h);
}
//--------------------------------------------------------

void 	glcd_draw_xy(unsigned char *img, 
												unsigned char x, unsigned char y) {
// y=byte addres
	unsigned char w,h;

	if(glcd_in_use==0u) return;
	w=*(img++);
	h=*(img++);
	
	do{
		glcd_xy(x,y++);
		glcd_ww=w;	
		do{
			glcd_write_data(*(img++) ^ glcd_gmodemask);
		} while(--glcd_ww);
	} while(--h);
}
//--------------------------------------------------------

void 				glcd_TextMode(unsigned char mode) {
//	glcd_gmodemask=0;
#ifndef GLCD_inverted
		glcd_gmodemask=0;
#endif
#ifdef GLCD_inverted
		glcd_gmodemask=255;
#endif

	if(mode==1u || mode==3u) //	glcd_gmodemask=255;
#ifndef GLCD_inverted
		glcd_gmodemask=255;
#endif
#ifdef GLCD_inverted
		glcd_gmodemask=0;
#endif
}
//--------------------------------------------------------

void 				glcd_PlotMode(unsigned char mode) {
	glcd_gmode=mode;
#ifdef GLCD_inverted
		if(mode==2u) glcd_gmode=3;
		if(mode==3u) glcd_gmode=2;
		if(mode==0u) glcd_gmode=1;
		if(mode==1u) glcd_gmode=0;
#endif
}
//--------------------------------------------------------

void 				glcd_print_char( char c) {
	unsigned int i;

	if(glcd_in_use==0u) return;
	i=((unsigned int)c*6)-192;
	c=7;
	while(--c){
		glcd_write_data(glcd_font[i++] ^ glcd_gmodemask);
	}
}
//--------------------------------------------------------

void 	glcd_print_xy_rom(rom const char *s, 
												unsigned char x, unsigned char y) {
	glcd_xy(x,y);
	glcd_puts_rom(s);
}
//--------------------------------------------------------

void 	glcd_print_xy_rom_v(rom const char *s, 
												unsigned char x, unsigned char y) {
	if(glcd_in_use==0u) return;	
	while(*s) {
		glcd_xy(x,y++);
		glcd_print_char(*(s++));
	}
}
//--------------------------------------------------------

void 	glcd_print_xy(char *s, unsigned char x, unsigned char y) {
	glcd_xy(x,y);
	glcd_puts(s);
}
//--------------------------------------------------------

void 	glcd_plot(unsigned char x, unsigned char y) {
// x,y pixel pos
 	unsigned char yy;
	unsigned char pix;
	yy=y;
	y>>=3;
	if(glcd_in_use==0u) return;
	if(x<glcd_clipx) return;
	if(x>=glcd_clipx2) return;
	if(y<glcd_clipy) return;
	if(y>=glcd_clipy2) return;
	
	glcd_xy(x,y);
	if(glcd_gmode>1u){
		pix=glcd_read_data();
		glcd_xy(x,y);
		if(glcd_gmode==2u)	glcd_write_data(  glcd_pixtab[yy&7] | pix );		
		if(glcd_gmode==3u)	glcd_write_data( (glcd_pixtab[yy&7] ^ 255) & pix );		
		if(glcd_gmode==4u)	glcd_write_data(  glcd_pixtab[yy&7] ^ pix );		
	}
	else {
		//	glcd_write_data((1<<(y&7)) ^ glcd_gmodemask);
		pix=glcd_pixtab[yy&7] ;
		if(glcd_gmode==1u) pix ^= 255 ;
		glcd_write_data( pix );
	}
}
//--------------------------------------------------------

void 	glcd_clear(unsigned char x, unsigned char y
										,unsigned char w, unsigned char h) {
// y, h =byte addres	
	if(glcd_in_use==0u) return;
	do{
		glcd_xy(x,y);
		y++;
		glcd_ww=w;
		do{
			glcd_write_data( glcd_gmodemask);
		} while(--glcd_ww);
	} while(--h);	
}
//--------------------------------------------------------

void 	glcd_clear_openclose(void) {
	glcd_open();
	glcd_clear_all();
	glcd_close();
}
//--------------------------------------------------------

void	glcd_clear_all(void) {
	#ifdef GLCD_Display128x64
		glcd_clear( 0, 0, 128, 8);
	#endif
	#ifdef GLCD_Display256x64
		glcd_clear( 0, 0, 256, 8);
	#endif
	#ifdef GLCD_Display128x128
		glcd_clear( 0, 0, 128, 16);
	#endif
}
//--------------------------------------------------------

void 	glcd_clip(unsigned char x, unsigned char y
										,unsigned char w, unsigned char h) {
// y, h =byte addres
	glcd_clipx=x;
	glcd_clipy=y;
	glcd_clipx2=x+w;
	glcd_clipy2=y+h;
}
//--------------------------------------------------------

void	glcd_ch_chip(void) {
			#ifdef GLCD_Display128x64
					if(glcd_hw_cfg.GLCD_SEL0==1u) {
						glcd_hw_cfg.GLCD_SEL0=0;
						glcd_hw_cfg.GLCD_SEL1=1;
					}
					else { 
						glcd_hw_cfg.GLCD_SEL1=0;
				//		glcd_hw_cfg.GLCD_SEL0=1
					}
			#endif
			#ifdef GLCD_Display256x64
				if(glcd_hw_cfg.GLCD_SEL0==1u) {
					glcd_hw_cfg.GLCD_SEL0=0;
					glcd_hw_cfg.GLCD_SEL1=1;
				}			
				else if(glcd_hw_cfg.GLCD_SEL1==1u) {
					glcd_hw_cfg.GLCD_SEL1=0;
					glcd_hw_cfg.GLCD_SEL2=1;
				}			
				else if(glcd_hw_cfg.GLCD_SEL2==1u) {
					glcd_hw_cfg.GLCD_SEL2=0;
					glcd_hw_cfg.GLCD_SEL3=1;
				}			
				else {
					glcd_hw_cfg.GLCD_SEL3=0;
				}			
			#endif
			#ifdef GLCD_Displayd128x128
					if(glcd_hw_cfg.GLCD_SEL0==1u) {
						glcd_hw_cfg.GLCD_SEL0=0;
						glcd_hw_cfg.GLCD_SEL1=1;
					}
					if(glcd_hw_cfg.GLCD_SEL2==1u) {
						glcd_hw_cfg.GLCD_SEL2=0;
						glcd_hw_cfg.GLCD_SEL3=1;
					}
					else { 
						glcd_hw_cfg.GLCD_SEL1=0;
						glcd_hw_cfg.GLCD_SEL3=0;
					}

			#endif
}
//--------------------------------------------------------

void glcd_fill_row(unsigned char x, unsigned char y
										,unsigned char w, unsigned char c){
	if(glcd_in_use==0u) return;
	glcd_xy(x,y);
	for( ; w>0u; w--){
		glcd_print_char( c );
	}										
}										
//--------------------------------------------------------
										
void glcd_fill_col(unsigned char x, unsigned char y
										,unsigned char h, unsigned char c){
	if(glcd_in_use==0u) return;
	h+=y;
	for( ; y<h; y++){
		glcd_xy(x,y);
		glcd_print_char( c );
	}										
}										
//--------------------------------------------------------
										
void glcd_pix_col(unsigned char x, unsigned char y
										,unsigned char h, unsigned char c){
	if(glcd_in_use==0u) return;
	h+=y;
	for( ; y<h; y++){
		glcd_xy(x,y);
		glcd_write_data(c ^ glcd_gmodemask);
	}										
}										
//--------------------------------------------------------
										
