// glcd.h

// ***************************************************************
// *               Copyright (C) Frank Linder                    *
// * Permission is granted to use, modify, or redistribute this  *
// * software so long as it is not sold or exploited for profit. *
// *    Provided "as is" without express or implied warranty.    *
// ***************************************************************

#ifndef glcg_h
#define glcg_h

// image format: width, height, data....

//----- defines ------------------------------------------
// #define GLCD_UseBusyFlag

#define GLCD_DATA_PORT PORTB
#define GLCD_TRIS_PORT TRISB

#define GLCD_Display128x64
//#define GLCD_Display256x64		// 240x64
//#define GLCD_Display128x128


#ifdef GLCD_Display128x64
// shiftreg bits
	#define GLCD_LED	b0
	#define GLCD_SEL1	b3
	#define GLCD_SEL0	b4
	#define GLCD_RS	b5
	#define GLCD_RW	b6
	
	#define GLCD_SELBITS_MASK 	 0b11100111
	#define GLCD_SELBITS_SELECT 0b00011000
	#define GLCD_SEL1B			 0b00001000
	#define GLCD_SEL0B			 0b00010000

	#define GLCD_BUSY RD7
	#define GLCD_DOFF RD5
	#define GLCD_RES  RD4

	#define GLCD_BUSYB 			 0b10000000
	#define GLCD_DOFFB 			 0b00100000
	#define GLCD_RESB  			 0b00010000
#endif

#ifdef GLCD_Display256x64
// shiftreg bits
	#define GLCD_LED	b0
	#define GLCD_SEL3	b1
	#define GLCD_SEL2	b2
	#define GLCD_SEL1	b3
	#define GLCD_SEL0	b4
	#define GLCD_RS	b5
	#define GLCD_RW	b6
	
	#define GLCD_SELBITS_MASK 	 0b11100001
	#define GLCD_SELBITS_SELECT 0b00011110
	#define GLCD_SEL3B			 0b00000010
	#define GLCD_SEL2B			 0b00000100	
	#define GLCD_SEL1B			 0b00001000
	#define GLCD_SEL0B			 0b00010000

	#define GLCD_BUSY RD7
	#define GLCD_DOFF RD5
	#define GLCD_RES  RD4

	#define GLCD_BUSYB 			 0b10000000
	#define GLCD_DOFFB 			 0b00100000
	#define GLCD_RESB  			 0b00010000
#endif

#ifdef GLCD_Display128x128
// shiftreg bits
	#define GLCD_LED	b0
	#define GLCD_SEL3	b1
	#define GLCD_SEL2	b2
	#define GLCD_SEL1	b3
	#define GLCD_SEL0	b4
	#define GLCD_RS	b5
	#define GLCD_RW	b6
	
	#define GLCD_SELBITS_MASK 	 0b11100001
	#define GLCD_SELBITS_SELECT 0b00011110
	#define GLCD_SEL3B			 0b00000010
	#define GLCD_SEL2B			 0b00000100	
	#define GLCD_SEL1B			 0b00001000
	#define GLCD_SEL0B			 0b00010000

	#define GLCD_BUSY  RD7
	#define GLCD_DOFF  RD5
	#define GLCD_RES   RD4

	#define GLCD_BUSYB 			 0b10000000
	#define GLCD_DOFFB 			 0b00100000
	#define GLCD_RESB  			 0b00010000
#endif


// commands
#define GLCD_ON	0b00111111
#define GLCD_OFF	0b00111110
#define GLCD_XPOS	0b01000000
#define GLCD_YPOS	0b10111000

//glcd_mode
//Text: 		GLCD_B_W   = GLCD_SET   = Black on White
//      		GLCD_W_B   = GLCD_CLEAR = White on Black
//      		GLCD_INV -> NO SUPPORT!

//
//Plotting:	GLCD_B_W   = Black on White
//      		GLCD_W_B   = White on Black
//      		GLCD_SET	  = Add a pixel
//      		GLCD_CLEAR = Remove a pixel
//      		GLCD_INV   = Invert a pixel

#define GLCD_inverted

#define GLCD_B_W		0 
#define GLCD_W_B  	1
#define GLCD_SET		2 
#define GLCD_CLEAR	3
#define GLCD_INV		4

//----- variables ----------------------------------------
extern unsigned char glcd_in_use;


//----- functions ----------------------------------------
	void 				glcd_init(void);
	unsigned char	glcd_open(void);
	void 				glcd_close(void);
	void 				glcd_write(unsigned char byte);
	void 				glcd_xy(unsigned char x, unsigned char y);
	void 				glcd_puts_rom(rom const char *s);
	void 				glcd_puts(unsigned char *s);
	void 				glcd_draw_xy_rom(rom const unsigned char *img, 
												unsigned char x, unsigned char y);
	void 				glcd_draw_xy(unsigned char *img, 
												unsigned char x, unsigned char y);
	void 				glcd_TextMode(unsigned char mode);
	void 				glcd_PlotMode(unsigned char mode);
	void 				glcd_print_char( char c);
	void 				glcd_print_xy_rom(rom const char *s, 
												unsigned char x, unsigned char y);
	void 				glcd_print_xy_rom_v(rom const char *s, 
												unsigned char x, unsigned char y);
	void 				glcd_print_xy(char *s, 
												unsigned char x, unsigned char y);
	void 				glcd_plot(unsigned char x, unsigned char y);
	void 				glcd_clear(unsigned char x, unsigned char y
										,unsigned char w, unsigned char h);
	void 				glcd_clear_all(void);
	void 				glcd_clear_openclose(void);
	void				glcd_clip(unsigned char x, unsigned char y
										,unsigned char w, unsigned char h);
	void				glcd_fill_row(unsigned char x, unsigned char y
										,unsigned char w, unsigned char c);
	void				glcd_fill_col(unsigned char x, unsigned char y
										,unsigned char h, unsigned char c);
	void				glcd_pix_col(unsigned char x, unsigned char y
										,unsigned char h, unsigned char c);
//--------------------------------------------------------
#endif
