// sampl.c

// ***************************************************************
// *               Copyright (C) Frank Linder                    *
// * Permission is granted to use, modify, or redistribute this  *
// * software so long as it is not sold or exploited for profit. *
// *    Provided "as is" without express or implied warranty.    *
// ***************************************************************

#include "main.h"
#include "sampl.h"
#include "keyb.h"
//----- defines ------------------------------------------


//----- function prototypes-------------------------------
void sampl_GetSingle(void);
void sampl_GetDouble(void);
void sampl_GetQuad(void);
void sampl_GetOct(void);
void sampl_GetHexdec(void);
void sampl_Get32(void);
void sampl_Get64_del(void);

//----- variables ----------------------------------------

// base sample speed / averange values stored at speed
//	153k/153k			153k/77k				153k/38k				153k/19.2k
//	153k/9.6k			153k/4804			77k/2404				38k/1202
// 19.2k/601			16.1k/252			8064/126				4032/63
//	2016/31.5			1008/15.7			504/7.9				252/3.94
// 126/1.97				63/0.98

void (*sampl_fpek[SAMPLE_NUM_MODES])(void)={
	sampl_GetSingle,	sampl_GetDouble,	sampl_GetQuad,		sampl_GetOct,
	sampl_GetHexdec,	sampl_Get32,		sampl_Get32,		sampl_Get32,
	sampl_Get64_del,	sampl_Get64_del,	sampl_Get64_del,	sampl_Get64_del,
	sampl_Get64_del,	sampl_Get64_del,	sampl_Get64_del,	sampl_Get64_del,
	sampl_Get64_del,	sampl_Get64_del
};
unsigned char sampl_ADC_FOSC[SAMPLE_NUM_MODES]={
	ADC_FOSC_16,		ADC_FOSC_16,		ADC_FOSC_16,		ADC_FOSC_16,
	ADC_FOSC_16,		ADC_FOSC_16,		ADC_FOSC_32,		ADC_FOSC_64,
	1,						1,						2,						4,							
	8,						16,					32,					64,					
	128,					0
};

rom unsigned char sampl_div[SAMPLE_NUM_MODES][4]={
	{  1,  1,  2,  2 },	{  1,  2,  2,  2 },	//	0,1
	{  2,  2,  2,  2 },	{  2,  2,  3,  3 },	//	2,3
		
	{  2,  3,  3,  3 },	{  3,  3,  3,  3 },	//	4,4
	{  3,  3,  3,  3 },	{  3,  4,  4,  4 },	//	6,7
		
	{  3,  4,  4,  4},	{  4,  4,  5,  5},	//	8,9
	{  4,  5,  6,  6},	{  5,  6,  6,  6},	//	10,11

	{  6,  6,  6,  6},	{  6,  6,  7,  7},	//	12,13
	{  6,  7,  7,  7},	{  7,  7,  7,  7},	//	14,15

	{  7,  7,  8,  8},	{  7,  8,  8,  8},	//	16,17
};

rom unsigned char sampl_str[9][6]={
	"100",	// 0 ???
	"0.1m",	// 1 
	"1.0m",	// 2
	" 10m",	// 3
	"100m",	// 4 ???
	" 0.1",	// 5
	"   1",	// 6
	"  10",	// 7
	" 100",	// 8
};

rom unsigned char sampl_step[SAMPLE_NUM_MODES][4]={
	//		/2	  /4  /6				rate     time		base div
	{ 31, 77, 154, 154 },	// 153k		6,54		200
	{ 38, 77, 154, 154 },	//  77k		13,07	0.5m
	{ 38, 77, 192, 192 },	//  38.4k	26,14	1m
	{ 38, 96, 192, 192 },	//  19.2k	52,3		2m
		
	{ 48, 96, 192, 192 },	//   9.6k	0,105m	5m
	{ 48, 96, 240, 240 },	//   4804	0,21m		10m
	{ 48,120, 240, 240 },	//   2402	0,42m		20m
	{ 24, 60, 120, 120 },	//   1200	0,84m		20m

	{ 30, 60, 120, 120 },	//   601		1,67m		50m
	{ 25, 50, 125, 125 },	//   252		3,97m		100m
	{ 25, 63, 133, 133 },	//   126		7,94m		200m
	{ 31, 63, 133, 133 },	//   63		15,9m		500m

	{ 31, 63, 157, 157 },	//   31,5	31,7m		1
	{ 31, 79, 157, 157 },	//   15,7	63,5m		2
	{ 39, 79, 157, 157 },	//   7,9		127m		5
	{ 39, 79, 197, 197 },	//   3,9		254m		10

	{ 39, 98, 197, 197 },	//   1,97	0,508m	20
	{ 49, 98, 197, 197 },	//   0,98	1,02		50
};

rom unsigned char sampl_cnt_div[SAMPLE_NUM_MODES][4]={
	{ 2, 5, 1, 1 },	// 0
	{ 5, 1, 2, 2 },	// 1
	{ 1, 2, 5, 5 },	// 2
	{ 2, 5, 1, 1 },	// 3
	 
	{ 5, 1, 2, 2 },	// 4
	{ 1, 2, 5, 5 },	// 5
	{ 2, 5, 1, 1 },	// 6
	{ 2, 1, 2, 2 },	// 7

	{ 5, 1, 2, 2 },	// 8
	{ 1, 2, 5, 5 },	// 9
	{ 2, 5, 1, 1 },	// 10
	{ 5, 1, 2, 2 },	// 11

	{ 1, 2, 5, 5 },	// 12
	{ 2, 5, 1, 1 },	// 13
	{ 5, 1, 2, 2 },	// 14
	{ 1, 2, 5, 5 },	// 15

	{ 2, 5, 1, 1 },	// 16
	{ 5, 1, 2, 2 },	// 17
};

rom unsigned char sampl_time_str[SAMPLE_NUM_MODES][11]={
//	 rate  time
	"153k 6,5s",
	"77k   13s",
	"38k   26s",
	"19k   52s",
		
	"9.6k 105s",
	"4.8k 210s",
	"2.4k 420s",
	"1.2k 840s",

	"600  1,7ms",
	"250    4ms",
	"125    8ms",
	"63    16ms",

	"32    32ms",
	"16    64ms",
	"8    127ms",
	"4    254ms",

	"2    508ms",
	"1       1s",
};
/*
// tid = (sampl_time_mul * x) / 2    1 <= x <=85
rom unsigned char sampl_time_mul[SAMPLE_NUM_MODES]={
	  13,  26,  52, 105, //  6.5 - 4992, 13 - 9984, 26 - 19968, 52 - 40000
	 210,  42,  84, 168, //  105 - 80640, 210 - 161280, 420 - 320000, 840 - 645000 
	  34,   8,  16,  32, //  1680 - 1290240, 4-3072, 8-6144, 16-12288 
	  64, 127, 254,   5, //  32-24576, 64-49152, 127-97536, 254-195072 
	  1,    2            //  500-390144, 1-768 
};
*/
// tid = (sampl_time_mul * x) >>sampl_time_shift
rom unsigned char sampl_time_mul[SAMPLE_NUM_MODES]={
	  13,  26,  52,  21, //  0.006-4.992,  0.013-9.984,  0.026-20.000,  0.05-40.00
	  42,  42,  84,  67, //  0.10- 80.64,  0.21-161.28,  0.42-320.00,   0.8-645.0 
	  67,   4,   8,  16, //  1.6- 1290.2,  0.004-3.072,  0.008-6.144,   0.016-12.288 
	  32,  51,  51,  51, //  0.032-24.576, 0.06-49.15,   0.12-97.53,    0.25-195.07 
	  41,  41            //  0.5-390.1,    1.0-768.0 
};

rom unsigned char sampl_time_shift[SAMPLE_NUM_MODES]={
	1, 1, 1, 2,
	2, 1, 1, 3,
	2, 0, 0, 0,
	0, 3, 2, 1,
	3, 2
};

rom unsigned char sampl_time_dot[SAMPLE_NUM_MODES]={
	3, 3, 3, 2,
	2, 2, 2, 1,
	1, 3, 3, 3,
	3, 2, 2, 2,
	1, 1	
};
// if sampl_mode<9 time="ms" else "s"

rom unsigned char sampl_trig_str[SAMPLE_NUM_TRIG][10]={
	"No trig  ", "Level +  ", "Level -  ", "Auto+ max", "Auto- min", 
	"Auto+ mid", "Auto- mid", 
};

rom unsigned char sampl_amp_str[SAMPLE_NUM_AMP][7]={
//	"-6..22", "-3..11","-1..5 ",  "1~   ", "0.2~ ", "0.1~ ", "50m~ "
	"-6..17", "-3..8","-1..4 ",  "1~   ", "0.2~ ", "0.1~ ", "50m~ "
};

rom unsigned char sampl_amp[SAMPLE_NUM_AMP]={
//	0,1,2,  0,1,3,5 // F=1,2,4 ,1,2,5,10 
	0,1,2,  0,2,4,6 // F=1,2,4 ,1,4,8,16
};

rom unsigned char sampl_amp_ch[SAMPLE_NUM_AMP]={
	2,2,2,  1,1,1,1
};

rom unsigned char sampl_amp_mul[SAMPLE_NUM_AMP]={
//	77,38,0,0,0,0
//	128,64,32,  128,64,0,0
	128,64,32,  128,32,16,8
};
/*
rom unsigned char sampl_amp_L0[SAMPLE_NUM_AMP]={
	30,60,121,  1,1,7,1
};
*/

/*
rom unsigned char sampl_amp_step[SAMPLE_NUM_AMP*4]={
//
	 31, 77, 154, 154 ,	//	15V	
	 38, 77, 154, 154 ,	//  5V	
	 38, 77, 192, 192 ,	//  5V~	
	 38, 96, 192, 192 ,	//	 2.5V~
								
	 48, 96, 192, 192 ,	//  1V~
	 48, 96,  96, 150 ,	// 
};
*/
/*
rom unsigned char sampl_amp_div[SAMPLE_NUM_AMP*4][6]={
	"1",	 "2",   "4",   "8",		// 15V
	"0.2", "0.5",   "1", "2",		//  5V
	"0.2", "0.5",   "1", "2",		//  5V~
	"0.1", "0.2", "0.5", "1",		//  2.5V~

	"50m", "0.1", "0.2", "0.5",	//  1V~
	"", "", "", "",	// 
};
*/

word sampl_sum;
word sampl_sum2;
unsigned char sampl_tmp;
unsigned char sampl_s;
unsigned char sampl_s2;
unsigned char sampl_mode;
unsigned char sampl_trglev;
unsigned char sampl_trgmode;
unsigned char sampl_amplifier;
unsigned char sampl_null;


//----- functions ----------------------------------------
void	sampl(void){
	unsigned char sampl_trg, sampl_tmode;
	unsigned char max;
	unsigned char min;
	
	void (*fpek)(void);
	//	~76000 samples/s  		~152000
	
	sampl_s=0;
	while(1) {
		shared1.predata[sampl_s]=0;
		data[sampl_s]=0;
		data2[sampl_s]=0;
		
		sampl_s++;
		if(sampl_s==0u) break;
	}	
		
	if(sampl_mode>=SAMPLE_NUM_MODES) sampl_mode=SAMPLE_NUM_MODES-1;
	fpek=sampl_fpek[sampl_mode];	
	sampl_s2=sampl_ADC_FOSC[sampl_mode]	;

	if(sampl_mode<8u) {	
		Andi_setup(sampl_ADC_FOSC[sampl_mode]);	
	}
	else if(sampl_mode==8u) {	
		Andi_setup(ADC_FOSC_64);	
	}
	else  {	
		Andi_high(ADC_CH1);	
	}
	
	ANDI_GO	// start conversion
	prepek=0;	
	datapek=0;	
	
	sampl_trg=sampl_trglev;
	sampl_tmode=sampl_trgmode;
	
	switch(sampl_trgmode){		
		case 3:	//"Auto+ max"
			sampl_tmode=1;
			max=0;
			while(1){
				if(KEYB_ESC)	break;				
				fpek();
				shared1.predata[prepek++]=sampl_sum.lu;
				if(sampl_sum.lu>max) max=sampl_sum.lu;
				if(prepek==0u) break;
			}		
			sampl_trg=max;
			break;
		case 4: // 	"Auto- min"
			sampl_tmode=2;
			min=255;
			while(1){
				if(KEYB_ESC)	break;				
				fpek();
				shared1.predata[prepek++]=sampl_sum.lu;
				if(sampl_sum.lu<min) min=sampl_sum.lu;
				if(prepek==0u) break;
			}		
			sampl_trg=max;
			break;		
		case 5: // 	"Auto+ mid"
			sampl_tmode=1;
			max=0;
			min=255;
			while(1){
				if(KEYB_ESC)	break;				
				fpek();
				shared1.predata[prepek++]=sampl_sum.lu;
				if(sampl_sum.lu<min) min=sampl_sum.lu;
				if(sampl_sum.lu>max) max=sampl_sum.lu;
				if(prepek==0u) break;
			}		
			sampl_trg=(max>>1) + (min>>1);
			break;		
		case 6: // 	"Auto- mid"
			sampl_tmode=2;
			max=0;
			min=255;
			while(1){
				if(KEYB_ESC)	break;				
				fpek();
				shared1.predata[prepek++]=sampl_sum.lu;
				if(sampl_sum.lu<min) min=sampl_sum.lu;
				if(sampl_sum.lu>max) max=sampl_sum.lu;
				if(prepek==0u) break;
			}		
			sampl_trg=(max>>1) + (min>>1) ;
			break;		
		default:
			// fill pre trig buffer
			while(1){
				if(KEYB_ESC)	break;				
				fpek();
				shared1.predata[prepek++]=sampl_sum.lu;
				if(prepek==0u) break;
			}		
	}

		
	
	if(sampl_tmode==1u){
	// wait for highlevel
		while(1){
			if(KEYB_ESC)	break;				
			fpek();
			shared1.predata[prepek++]=sampl_sum.lu;
			if( sampl_sum.lu<sampl_trg ) break;			
		}
	
	// wait for low(trig)level
		while(1){
			if(KEYB_ESC)	break;				
			fpek();
			shared1.predata[prepek++]=sampl_sum.lu;
			if(sampl_sum.lu>sampl_trg) break;			
		}
	}	
	
	if(sampl_tmode==2u) {
	// wait for highlevel
		while(1){
			if(KEYB_ESC)	break;				
			fpek();
			shared1.predata[prepek++]=sampl_sum.lu;
			if( sampl_sum.lu>sampl_trg ) break;			
		}
	
	// wait for low(trig)level
		while(1){
			if(KEYB_ESC)	break;				
			fpek(); 
			shared1.predata[prepek++]=sampl_sum.lu;
			if( sampl_sum.lu<sampl_trg ) break;			
		}
	}	
	
// sampling 1
	while(1){		
		if(KEYB_ESC)	break;				
		fpek();
		data[datapek++]=sampl_sum.lu;	
		if(datapek==0u) break;			
	}
// sampling 2
	while(1){
		if(KEYB_ESC)	break;				
		fpek();
		data2[datapek++]=sampl_sum.lu;	
		if(datapek==0u) break;			
	}
}
//--------------------------------------------------------

void sampl_GetSingle(void) {
	while( ANDI_BUSY );
	ANDI_GO	// start conversion
	sampl_sum.lu=ADRESH;
}
//--------------------------------------------------------

void sampl_GetDouble(void) {
	sampl_sum.iu=0;
	for(sampl_s=2; sampl_s>0u;sampl_s--) {	
		while( ANDI_BUSY );
		ANDI_GO	// start conversion
		sampl_sum.iu+=ADRESH;
	}
	sampl_sum.iu>>=1;	
}
//--------------------------------------------------------

void sampl_GetQuad(void) {
	sampl_sum.iu=0;
	for(sampl_s=4; sampl_s>0u;sampl_s--) {	
		while( ANDI_BUSY );
		ANDI_GO	// start conversion
		sampl_sum.iu+=ADRESH;
	}
	sampl_sum.iu>>=2;	
}
//--------------------------------------------------------

void sampl_GetOct(void) {	
	sampl_GetQuad();
	sampl_sum2.iu=sampl_sum.iu;
	sampl_GetQuad();
	sampl_sum.iu+=sampl_sum2.iu;
	sampl_sum.iu>>=1;
}
//--------------------------------------------------------

void sampl_GetHexdec(void) {
	sampl_sum.iu=0;
	for(sampl_s=16; sampl_s>0u;sampl_s--) {	
		while( ANDI_BUSY );
		ANDI_GO	// start conversion
		sampl_sum.iu+=ADRESH;
	}
	sampl_sum.iu>>=4;	
}
//--------------------------------------------------------

void sampl_Get32(void) {
	sampl_GetHexdec();
	sampl_sum2.iu=sampl_sum.iu;
	sampl_GetHexdec();
	sampl_sum.iu+=sampl_sum2.iu;
	sampl_sum.iu>>=1;
}
//--------------------------------------------------------

void sampl_Get64_del(void) {
	// ADC_FOSC_64 asumed!  		18.5k/64 = 289  252
	// del =1	252		// del =2	126		// del =4	63
	// del =8	31.5		// del =16	15.75		// del =32	7.88	
	// del =64	3.94		// del =128	1.97		// del =0	0.98	

	sampl_sum.iu=0;
	for(sampl_s=64; sampl_s>0u;sampl_s--) {			
		sampl_tmp=sampl_s2;			
		while(1) {
			while( ANDI_BUSY );
			ANDI_GO	// start conversion
			sampl_tmp--;
			if(sampl_tmp==0u) break;
		}
		sampl_sum.iu+=ADRESH;
	}
	sampl_sum.iu>>=6;	
//	sampl_sum.iu/=(unsigned int)64;	
}
//--------------------------------------------------------
