// SPI_Multiple_Channel driver  

// ***************************************************************
// *               Copyright (C) Frank Linder                    *
// * Permission is granted to use, modify, or redistribute this  *
// * software so long as it is not sold or exploited for profit. *
// *    Provided "as is" without express or implied warranty.    *
// ***************************************************************

#include "main.h"
#include <spi.h>
#include "spi_mc.h"

//----- variables ----------------------------------------
rom spi_datastruct *spi_dev_desc;
volatile unsigned char spi_tmp_reg;  
//--------------------------------------------------------

//----- functions ----------------------------------------
void	spi_init(void) {
	spi_dev_desc=0;
}
//--------------------------------------------------------

unsigned char spi_open(rom spi_datastruct *spi_desc) {
	if(spi_dev_desc!=0) return 0;
	spi_dev_desc=spi_desc;
	OpenSPI(spi_dev_desc->speed, spi_dev_desc->mode, spi_dev_desc->samp);

	spi_tmp_reg= *(spi_dev_desc->port);
	if(spi_dev_desc->time.b1)
		spi_tmp_reg |= spi_dev_desc->mask;
	else
		spi_tmp_reg &= (~spi_dev_desc->mask);
	*spi_dev_desc->port=spi_tmp_reg;
	return 1;
}
//--------------------------------------------------------  

unsigned char	spi_isopen(void) {
	if(spi_dev_desc) return 1;
	return 0;
}
//--------------------------------------------------------  

volatile void spi_short_delay(void) {
	spi_tmp_reg&=255;
}
//--------------------------------------------------------  

void spi_strobe(char tx) {
	if(spi_dev_desc==0u) return;
   if(tx&1){
		spi_tmp_reg= *(spi_dev_desc->port);
		if(spi_dev_desc->time.b3)
			spi_tmp_reg |= spi_dev_desc->mask;
		else
			spi_tmp_reg &= (~spi_dev_desc->mask);
		*(spi_dev_desc->port)=spi_tmp_reg;
	}                  
	
	spi_short_delay(); 	
	
   if(tx&2){
		spi_tmp_reg= *(spi_dev_desc->port);
		if(spi_dev_desc->time.b0)
			spi_tmp_reg |= spi_dev_desc->mask;
		else
			spi_tmp_reg &= (~spi_dev_desc->mask);
		*(spi_dev_desc->port)=spi_tmp_reg;
	}
}
//--------------------------------------------------------

void spi_close(void) {
	if(spi_dev_desc==0u) return;

	spi_tmp_reg= *(spi_dev_desc->port);
	if(spi_dev_desc->time.b2)
		spi_tmp_reg |= spi_dev_desc->mask;
	else
		spi_tmp_reg &= (~spi_dev_desc->mask);
	*(spi_dev_desc->port)=spi_tmp_reg;

	spi_short_delay();

//	spi_tmp_reg= *(spi_dev_desc->port);
	if(spi_dev_desc->time.b0)
		spi_tmp_reg |= spi_dev_desc->mask;
	else
		spi_tmp_reg &= (~spi_dev_desc->mask);
	*(spi_dev_desc->port)=spi_tmp_reg;

	spi_dev_desc=0u;
}
//--------------------------------------------------------

void spi_write(unsigned char byte) {
	if(spi_dev_desc==0u) return;
	spi_tmp_reg=SSPBUF;
	SSPBUF = byte;					// write byte
	do{
		spi_tmp_reg=SSPSTAT;
	} while( (spi_tmp_reg&1u)==0u );	// wait
}
//--------------------------------------------------------

unsigned char	spi_read(void) {
	if(spi_dev_desc==0u) return 0;
	spi_tmp_reg=SSPBUF;
	SSPBUF = 0x00;					// send dummy       
	while ( (SSPSTAT&1u)==0u );	// wait
	return ( SSPBUF );			// return with byte read    
}
//--------------------------------------------------------

