// SPI_Multiple_Channel driver

// ***************************************************************
// *               Copyright (C) Frank Linder                    *
// * Permission is granted to use, modify, or redistribute this  *
// * software so long as it is not sold or exploited for profit. *
// *    Provided "as is" without express or implied warranty.    *
// ***************************************************************     

#include "main.h"
#include "spi.h"

#ifndef SPI_MC_H
#define SPI_MC_H

#define SPI_ULTRA SPI_FOSC_4
#define SPI_FAST  SPI_FOSC_16
#define SPI_SLOW  SPI_FOSC_64

#define SPI_STROBE_SET	1
#define SPI_STROBE_RST	2
#define SPI_STROBE		3


//                            |<---Open--->|<Close>|
//----- timing -------------t0------t1-------t2-t0-----------
//									  .       .        .  .
// +pulse when close			 ________________||___ shift_reg
//									  .       .        .  .
// -pulse when close			 |__|
//									  .       .        .  .
// -when open / +when close |________________| SPI_device (EEPROM, AD/DA-converter..)
//									  .       .        .  .
// +when open / -when close __||___
//									  .       .        .  .
//-----------------------------------------------------------
//
//                              strobe
//----- timing -------------t0----t3----t0--
//									  .     .     .
// +pulse when strobe		 _____||______     shift_reg
//									  .     .     .
// -pulse when strobe		 |__|
//									  .     .     .


typedef struct {
	volatile unsigned char *port;
	volatile unsigned char *tris;
	unsigned char 		mode;
	unsigned char 		speed;
	unsigned char 		mask; // bit-mask, ex. bit0->0x01, bit1->0x02, bit3->0x04...
	bituchar				time; // bit 0=t0 bit 1=t1, bit2=t2, bit3=t3
	unsigned char 		samp;
} spi_datastruct; 

/*
	timing example     
	--------------------------------------------------------
                //  bit:  3   2   1   0  
					 //  time: t3  t2  t1  t0  
	SPI_CH_DISPLAY 0   //  1   0   0   0    //0x08
	SPI_CH_EEPROM  1   //  0   0   0   1    //0x01
	SPI_CH_mcp6sx  2   //  0   0   0   1    //0x01
	SPI_CH_4094    4   //  1   1   0   0    //0x0c
*/

/*
	spi_datastruct example  
	--------------------------------------------------------
rom spi_datastruct G_disp=
	{&PORTE, &DDRE, MODE_00, spi_ultra, 0x01, 0x08, SMPMID };	//  SPI_CH_DISPLAY,  bit 0	
	
rom spi_datastruct eeprom_desc=
	{&PORTA, &DDRA, MODE_00, SPI_FAST , 0x20, 0x01, SMPMID };	//  SPI_EEPROM, portA, bit 5

rom spi_datastruct mcp6sx_desc=
	{&PORTE, &DDRE, MODE_00, SPI_SLOW , 0x02, 0x01, SMPMID };	//  MCP6SX,     portE, bit 1
	
a shift reg like 4094:         
rom spi_datastruct IC4094_desc=
	{&PORTA, &DDRA, MODE_00, SPI_SLOW , 0x10, 0x0c, SMPMID };	//  SPI_CH_1,  bit 4	
*/                                        


//----- functions ----------------------------------------
	void 				spi_init(void);   
	unsigned char	spi_isopen(void);   
   void 				spi_strobe(char tx);                        
	unsigned char  spi_open(rom spi_datastruct *spi_desc);   
	void           spi_close(void);
	void 				spi_write(unsigned char byte);
	unsigned char	spi_read(void);

//--------------------------------------------------------        


#endif
